$ifndef _USART1_Buffer24_inc_
$define _USART1_Buffer24_inc_
'
' Buffered USART receiver that replaces the compiler's Hrsin command library routines, both with and without timeout
'
' For use with the Proton24 compiler version 1.0.1.0 onwards
'
' The compiler will produce the following hooks for use with #ifSym:
' __SYSCOM_HRSIN1_REQ_      for the HRSIn command without timeout
' __SYSCOM_HRSIN1_TO_REQ_   for the HRSIn command with timeout
'
    #disable HRSIn1, HRsin1To                           ' Disable the compiler's Hrsin (with and without timeout) library routines
'
' Default size of the RX buffer
'
$ifndef cUSART_RXBufferSize
    $define cUSART_RXBufferSize 64      
$endif  
'
' Create some variable for use with the interrupt buffer
'    
    Dim _USART1_bBuffer[cUSART_RXBufferSize] As Byte    ' The RX buffer
    Dim _USART1_wIndexIn As Word                        ' Data in pointer
    Dim _USART1_wIndexOut As Word                       ' Data out pointer
    Dim USART1_tOverrun As Bit                          ' Buffer Overrun flag

    $define cHrsin1_TimeoutPadding $eval ((113 * _xtal) / 2) - 1 ' Hrsin1 timeout delay padding    
'-------------------------------------------------------------------------------------------
' Enable the USART1 Interrupt                                    
'
$define USART1_IsrEnable() Set IEC0bits_U1RXIE

'-------------------------------------------------------------------------------------------
' Disable the USART1 interrupt
'
$define USART1_IsrDisable() Clear IEC0bits_U1RXIE

'-------------------------------------------------------------------------------------------
' Reset the buffer variables                              
'
Proc USART1_BufferReset() 
    USART1_IsrDisable()                             ' Disable the USART1 interrupt
    USART1_tOverrun = 0                             ' Reset the Overrun flag
    Clear _USART1_bBuffer                           ' Clear the buffer
    Clear _USART1_wIndexIn                          ' \
    Clear _USART1_wIndexOut                         ' / Reset the data in and out pointers
    Clear IFS0bits_U1RXIF                           ' Clear the interrupt request flag
    USART1_IsrEnable()                              ' Enable the USART1 interrupt
EndProc

'-------------------------------------------------------------------------------------------
' Initialise the buffer and the USART1 interrupt handler                               
'
Proc USART1_Init()   
    USART1_IsrDisable()                             ' Disable the USART1 interrupt
    USART1_tOverrun = 0                             ' Reset the Overrun flag
    Clear _USART1_bBuffer                           ' Clear the buffer
    Clear _USART1_wIndexIn                          ' \
    Clear _USART1_wIndexOut                         ' / Reset the data in and out pointers    
    IPC2bits_U1RXIP0 = 1                            ' \
    IPC2bits_U1RXIP1 = 1                            ' | USART1 interrupt priority
    IPC2bits_U1RXIP2 = 1                            ' /
    Clear IFS0bits_U1RXIF                           ' Clear the interrupt request flag
    USART1_IsrEnable()                              ' Enable the USART1 interrupt
'
'----------------------------------------------------
' USART1 interrupt handler to buffer incoming data
'
Isr U1RXInterrupt
    #ifSym __SYSCOM_HSERIAL1_CLEAR_REQ_             ' Has the HSERIAL_CLEAR declare been used in the program?
        If U1STAbits_OERR = 1 Then                  ' Yes. So is there an overrun error?
            Clear U1STAbits_OERR                    ' Yes. So clear it
            GoTo Exit_USART1_Interrupt              ' Aad exit the interrupt
        EndIf
    #endIfSym
    _USART1_bBuffer[_USART1_wIndexIn] = U1RXREG     ' Read a data byte from the USART
    Inc _USART1_wIndexIn                            ' Move up the buffer
    If _USART1_wIndexIn > cUSART_RXBufferSize Then  ' Has the buffer been filed?
        Clear _USART1_wIndexIn                      ' Yes. So reset the in pointer
    EndIf
'
' Check for a buffer overrun. Flag USART1_tOverrun must be cleared in the main program 
'   
    If _USART1_wIndexIn = _USART1_wIndexOut Then USART1_tOverrun = 1    
Exit_USART1_Interrupt:
    Clear IFS0bits_U1RXIF                           ' Clear the interrupt request flag
EndIsr                                              ' Exit the interrupt handler
EndProc	   
'----------------------------------------------------------------------------
#ifSym __SYSCOM_HRSIN1_REQ_, __SYSCOM_HRSIN1_TO_REQ_ 
    GoTo _BufferedHrsin1_Main
#endIfSym
'----------------------------------------------------------------------------
' Hrsin replacement code with timeout
' Input     : PRTA1 = timeout value
' Output    : WREG0.Byte0
'           : Carry flag clear if timed out
' Notes     : Follows exactly the method used by the compiler's library routine
'
#ifSym __SYSCOM_HRSIN1_TO_REQ_
__hrsin1__to:
    Input __HRSIN1_PORT.__HRSIN1_PIN            ' Make the Hrsin1 pin an input (just in case it is altered elsewhere)    
    Push.d WREG2                                ' Save WREG2 and WREG3
    WREG2 = PRTA1                               ' Move the timeout value (passed from the command) to WREG2
    Clear WREG3                                 ' Clear the inner loop counter
    While                                       ' Create an infinite loop
        DelayCS 1                               ' Padding for 1 cycle
        If _USART1_wIndexIn <> _USART1_wIndexOut Then ' Is there any data in the buffer?
            Pop.d WREG2                         ' Yes. So restore WREG2 and WREG3
            GoTo __hrsin1__getbyte              ' Get the byte from USART1, and exit the routine from there
        EndIf
        Clear WREG0.Byte0                       ' \
        Dec WREG3.Byte0                         ' | Decrement the inner loop counter
        Subb.b WREG3H                           ' / So that it will trigger a Carry change
        If SRbits_C = 1 Then Continue           ' Continue if gone below zero?
        Dec WREG2.Byte0                         ' \ Decrement the outer loop counter
        Subb.b WREG2H                           ' / So that it will trigger a Carry change
        If SRbits_C = 0 Then                    ' Is the carry flag clear?
            Pop.d WREG2                         ' Yes. So restore WREG2 and WREG3
            Return                              ' Exit the routine
        EndIf
        WREG3 = cHrsin1_TimeoutPadding          ' Reload the timout padding into WREG3
    Wend                                        ' Look again
#endIfSym
'    
'----------------------------------------------------------------------------     
' Hrsin replacement code without timeout
' Input     : None
' Output    : WREG0.Byte0
' Notes     : None
'
#ifSym __SYSCOM_HRSIN1_REQ_
__hrsin1:
    Input __HRSIN1_PORT.__HRSIN1_PIN                ' Make the Hrsin1 pin an input (just in case it is altered elsewhere)
    While _USART1_wIndexIn = _USART1_wIndexOut      ' Wait for data in the buffer
    Wend
'
' Fall through __hrsin1__getbyte
'
#endIfSym    
'----------------------------------- 
' Get a single byte from the buffer and place it into WREG0.Byte0
' 
#ifSym __SYSCOM_HRSIN1_REQ_, __SYSCOM_HRSIN1_TO_REQ_ 
__hrsin1__getbyte:
    WREG2.Byte0 = _USART1_bBuffer[_USART1_wIndexOut]    ' Place a buffer byte into WREG2.Byte0
    Inc _USART1_wIndexOut                               ' Move up the buffer
    If _USART1_wIndexOut > cUSART_RXBufferSize Then     ' Is the out index overrunning the buffer size?
        Clear _USART1_wIndexOut                         ' Yes. So reset it
    EndIf 
    WREG0.Byte0 = WREG2.Byte0                           ' Transfer the byte from the buffer to WREG0
    Set SRbits_C                                        ' Set the carry flag to indicate byte received
    Return                                              ' Exit the routine
#endIfSym
'----------------------------------------------------------------------------
_BufferedHrsin1_Main:
    USART1_Init()                                       ' Initialise the USART1 buffer interrupt
    
$endif  ' _USART1_Buffer24_inc_

  

